#include "MPEGAStreamer.h"
#include <iostream>

using namespace std;

static const char *subversion_id = "$Id: MPEGAStreamer.cpp 102 2003-07-03 23:49:38Z  $";

using namespace MIMD;

MPEGAStreamer::~MPEGAStreamer() 
{
   if(audioSource) Medium::close(audioSource);
   if(mpegDemux) Medium::close(mpegDemux);
}

bool MPEGAStreamer::moveToNextSource() 
{
   const Source &s = getSource();

   if(beforeFirst) {
      curItem = s.getPlayList().begin();
      beforeFirst = false;
   } else
      curItem++;

   // Check to see if we already supplied a source
   if(audioSource) Medium::close(audioSource);
   if(mpegDemux) Medium::close(mpegDemux);

   audioSource = NULL;
   fileSource = NULL;
   mpegDemux = NULL;

   // Do the actual move

   if(curItem == s.getPlayList().end()) {
      if(s.isLooping()) 
         curItem = s.getPlayList().begin();
      else 
         return false;
   }

   // Figure out what the input is:
   MediaType::MediaDescriptor stype = curItem->getType();
   switch(stype.type)
   {
      // Special handling for layer 3 audio
      case MediaType::MPEG1L3:
      case MediaType::MPEG2L3:
         cerr << "MP3: " << curItem->getFileName() << endl;
         audioSource = MP3FileSource::createNew(*getEnvironment(), 
                                               curItem->getFileName().c_str());
         break;
      case MediaType::MPEGSS:
         cerr << "system stream: " << curItem->getFileName() << endl;
         // Must Demux
         fileSource = ByteStreamFileSource::createNew(*getEnvironment(), 
                                             curItem->getFileName().c_str());
         mpegDemux = MPEG1or2Demux::createNew(*getEnvironment(), fileSource);
         audioSource = MPEG1or2AudioStreamFramer::createNew(*getEnvironment(), 
                                             mpegDemux->newAudioStream());
         break;
      case MediaType::MPEG1L2: // Plain old layer 2 audio
      case MediaType::MPEG2L2:
         cerr << "Layer 2 audio: " << curItem->getFileName() << endl;
         ByteStreamFileSource *fileSource;
         fileSource = ByteStreamFileSource::createNew(*getEnvironment(), 
                                             curItem->getFileName().c_str());
         audioSource = MPEG1or2AudioStreamFramer::createNew(*getEnvironment(), 
                                                        fileSource);
         break;
      default:
         cerr << "MIMD: Audio source " << s.getName() << " contains no audio!"
              << endl << 
              "This should be reported as an internal bug in MIMD." << endl;
         break;
   }
   return true;
}
