#include "MPEGSSStreamer.h"

static const char *subversion_id = "$Id: MPEGSSStreamer.cpp 290 2004-03-25 18:50:42Z tkay $";

using namespace MIMD;

MPEGSSStreamer::~MPEGSSStreamer() 
{
   if(audioSource) Medium::close(audioSource);
   if(videoSource) Medium::close(videoSource);
   if(mpegDemux) Medium::close(mpegDemux);
   // The fileSource is auto-closed by these
}

bool MPEGSSStreamer::moveToNextSource() 
{
   const Source &s = getSource();

   if(beforeFirst) {
      curItem = s.getPlayList().begin();
      beforeFirst = false;
   } else
      curItem++;

   // Check to see if we already supplied a source
   if(audioSource) Medium::close(audioSource);
   if(videoSource) Medium::close(videoSource);
   if(mpegDemux) Medium::close(mpegDemux);
   // The fileSource is auto-closed by these

   audioSource = NULL;
   videoSource = NULL;
   mpegDemux = NULL;

   // Do the actual move

   if(curItem == s.getPlayList().end()) {
      if(s.isLooping()) 
         curItem = s.getPlayList().begin();
      else 
         return false;
   }

   fileSource = ByteStreamFileSource::createNew(*getEnvironment(), 
                                    curItem->getFileName().c_str());

   if(curItem->hasAudio() && curItem->hasVideo())
      mpegDemux = MPEG1or2Demux::createNew(*getEnvironment(), fileSource);

   if (s.isVideoDesired() && s.getVP()) {
      if (mpegDemux) // Do we need to split?
         videoSource = MPEG1or2VideoStreamFramer::createNew(*getEnvironment(), 
                                             mpegDemux->newVideoStream());
      else
         videoSource = MPEG1or2VideoStreamFramer::createNew(*getEnvironment(), fileSource);
   }

   if (mpegDemux != NULL && s.isAudioDesired() && s.getAP()) {
      audioSource = MPEG1or2AudioStreamFramer::createNew(*getEnvironment(), 
                                             mpegDemux->newAudioStream());
   }

   return true;
}
