#ifndef MIMD_MPEGSSS
#define MIMD_MPEGSSS

#include "parser.h"
#include <BasicUsageEnvironment.hh>
#include <RTPSink.hh>
#include <FramedSource.hh>
#include <MPEG1or2DemuxedElementaryStream.hh>
#include <MPEG1or2Demux.hh>
#include <MPEG1or2AudioRTPSink.hh>
#include <MPEG1or2VideoRTPSink.hh>
#include <MPEG1or2VideoStreamFramer.hh>
#include <MPEG1or2AudioStreamFramer.hh>
#include <Groupsock.hh>
#include <GroupsockHelper.hh>
#include <RTCP.hh>
#include <ByteStreamFileSource.hh>
#include <MediaSource.hh>
#include "MediaStreamer.h"

namespace MIMD
{
   /**
    * This class is meant for streaming MPEG system streams (A/V).
    */
   class MPEGSSStreamer : public MediaStreamer
   {
      public:
         /**
          * Create a new streamer using parameters from a configuration
          * Source object.
          * @param MIMD::Souce The configuration for the streamer.
          */
         MPEGSSStreamer(const Source &src) : MediaStreamer(src) {
            beforeFirst = true;
            fileSource = NULL;
            mpegDemux = NULL;
            audioSource = NULL;
            videoSource = NULL;
            audioSink = NULL;
            videoSink = NULL;
         }
         virtual ~MPEGSSStreamer();

         /**
          * Methods overriding MediaStreamer. @see MIMD::MediaStreamer.
          */
         virtual bool hasAudio() { return true; }
         virtual bool hasVideo() { return true; }
         virtual FramedSource *getAudioFSource() { return audioSource; }
         virtual FramedSource *getVideoFSource() { return videoSource; }
         virtual bool moveToNextSource();
      private:
         list<MediaItem>::const_iterator curItem;
         MPEG1or2Demux *mpegDemux;
         FramedSource *audioSource;
         FramedSource *videoSource;
         ByteStreamFileSource *fileSource;
         RTPSink *audioSink;
         RTPSink *videoSink;
         bool beforeFirst;
   };

};

#endif

