#ifndef MIMD_MPEGVS
#define MIMD_MPEGVS

#include "parser.h"
#include <BasicUsageEnvironment.hh>
#include <RTPSink.hh>
#include <FramedSource.hh>
#include <MPEG1or2DemuxedElementaryStream.hh>
#include <MPEG1or2Demux.hh>
#include <MPEG1or2AudioRTPSink.hh>
#include <MPEG1or2VideoRTPSink.hh>
#include <MPEG1or2VideoStreamFramer.hh>
#include <MPEG1or2AudioStreamFramer.hh>
#include <Groupsock.hh>
#include <GroupsockHelper.hh>
#include <RTCP.hh>
#include <ByteStreamFileSource.hh>
#include <MediaSource.hh>
#include "MediaStreamer.h"

namespace MIMD
{
   /**
    * This class is meant for streaming MPEG video-only streams.
    */
   class MPEGVStreamer : public MediaStreamer
   {
      public:
         /**
          * Create a new streamer using parameters from a configuration
          * Source object.
          * @param MIMD::Souce The configuration for the streamer.
          */
         MPEGVStreamer(const Source &src) : MediaStreamer(src) {
            beforeFirst = true;
            videoSource = NULL;
            mpegDemux = NULL;
            videoSink = NULL;
            fileSource = NULL;
         }
         virtual ~MPEGVStreamer();

         /**
          * Methods overriding MediaStreamer. @see MIMD::MediaStreamer.
          */
         virtual bool hasAudio() { return false; }
         virtual bool hasVideo() { return true; }
         virtual FramedSource *getAudioFSource() { return NULL; }
         virtual FramedSource *getVideoFSource() { return videoSource; }
         virtual bool moveToNextSource();
      private:
         list<MediaItem>::const_iterator curItem;
         MPEG1or2Demux *mpegDemux; // May need if the source is multiplexed
         FramedSource *videoSource;
         ByteStreamFileSource *fileSource;
         RTPSink *videoSink;
         bool beforeFirst;
   };
};

#endif
