#ifndef MIMD_MEDIATYPE
#define MIMD_MEDIATYPE
#include <string>
#include <stdexcept>

/**
 * A simple media type checker. This code is based on the description of
 * file magic numbers in magic(5). It currently supports a lot more the MIMD
 * needs, and much less than is out there.
 */
namespace MediaType
{
   class MediaTypeError : public std::invalid_argument
   {
    public:
      MediaTypeError(const std::string &msg) : std::invalid_argument(msg) {}
   };

   /* According to /usr/share/magic, MPEG?L? are audio only. Animation files
    * should show up as MPEGVS and MPEGSS.
    */
   enum MediaID { Unknown, MPEG1L3, MPEG1L2, MPEG2L3, MPEG2L2, MPEGVS,
                  MPEGSS, QTMOV, QTMDAT, WAV, AVI, NoFile };
   struct MediaDescriptor
   {
      MediaDescriptor() : type(Unknown), hasID3(false) {}
      MediaID type;
      bool hasID3;
   };

   // Check a specific file, and return a MediaID
   MediaDescriptor check(const char *filename);
   // Get a text description of a MediaID
   std::string getDescription(MediaID id);
};

#endif
