/*
 * mimd 0.2 Abiel Reinhart abielr@myrealbox.com
 */

#include <string>
#include <vector>
#include <pthread.h>
#include <unistd.h>
#include <errno.h>
#include "MediaType.h"
#include "MediaStreamer.h"
#include "MPEGSSStreamer.h"
#include "MPEGVStreamer.h"
#include "MPEGAStreamer.h"
#include "parser.h"


#ifdef SYSCONFDIR
#define xstr(s) str(s)
#define str(s) #s
#define MIMD_CONFIG_FILE xstr(SYSCONFDIR/mimd.xml)
#else
#define MIMD_CONFIG_FILE "/etc/mimd/mimd.xml"
#endif

using namespace MIMD;

static const char *subversion_id = "$Id: mimd.cpp 304 2004-03-25 19:52:41Z tkay $";
static char *config_file = MIMD_CONFIG_FILE;

void *threadEntry(void *arg)
{
   MediaStreamer *obj = (MediaStreamer *)arg;
   obj->run();
   pthread_exit(NULL);
   return NULL; // Unreachable, but avoid the warning
}

int main(int argc, char *argv[]) 
{
   MIMD::MIMDConfiguration configuration;
   pthread_t threadid;
	// std::vector<pthread_t> threads;
   int opt;
   extern char *optarg;

   while((opt=getopt(argc, argv, "vf:")) != -1)
   {
      switch(opt)
      {
         case '?':
         case 'v':
            cerr << "MIMD Version " << VERSION <<  endl;
            cerr << "   " << subversion_id << endl << endl;
            cerr << "Usage: " << argv[0] << " [-f configfile]" << endl;
            cerr << "   Default configuration file: " << config_file << endl;
            return -1;
            break;
         case 'f':
            config_file = optarg;
            cerr << "Configuration file set to " << optarg << endl;
            break;
      }
   }

   try {
      XMLPlatformUtils::Initialize();
      SAXParser* parser = new SAXParser;

      parser->setValidationScheme(SAXParser::Val_Always);
      parser->setDoNamespaces(false);
      parser->setDoSchema(true);
      parser->setValidationSchemaFullChecking(true);

      parser->setDocumentHandler(&configuration);
      parser->setErrorHandler(&configuration);

      parser->parse(config_file);

      if(configuration.isGoodParse() == false) {
         cerr << "MIMD: Exiting due to configuration errors." << endl;
         return -1;
      }

      delete parser;
      XMLPlatformUtils::Terminate();
   } catch(exception &e) {
      cerr << "Error reading configuration: " << e.what() << endl;
      return -1;
   }

   cerr << "MIMD: Streaming..." << endl;
   const list<Source*> &items = configuration.getSources();
   list<Source*>::const_iterator i;
   for(i = items.begin(); i != items.end(); i++)
   {
      Source *src = *i;
      MediaStreamer *theStreamer = NULL;
      if(src->isAudioDesired() && src->isVideoDesired())
         theStreamer = new MPEGSSStreamer(*src);
      else if(src->isVideoDesired())
         theStreamer = new MPEGVStreamer(*src);
      else if(src->isAudioDesired())
         theStreamer = new MPEGAStreamer(*src);
      else
         cerr << "MIMD: Error processing: " << src->getName() << 
              ". No audio of video streamer was requested." << endl;

      if(theStreamer == NULL) {
         cerr << "MIMD: Error: Could not create a streamer for " << 
              src->getName() << ". Skipping." << endl;
         continue;
      }
		pthread_create(&threadid, NULL, threadEntry, (void *)theStreamer);
      //pthread_detach(threadid);
   }

   pthread_exit(NULL);

	return 0;
}
